"""Root metrics for Bright-lite"""
from cymetric.root_metrics import root_metric


agentstate_brightlite_fuelfabfacilityinfo = root_metric(
    name='AgentState_Brightlite_FuelfabFacilityInfo')
agentstate_brightlite_reactorfacilityinfo = root_metric(
    name='AgentState_Brightlite_ReactorFacilityInfo')
agentstate_brightlite_reprocessfacilityinfo = root_metric(
    name='AgentState_Brightlite_ReprocessFacilityInfo')
brightlite_reactor_data = root_metric(
    name='BrightLite_Reactor_Data')
